/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  base64.cc - Simple BASE64 decoding.
 *
 *  GNU MP3D - A portable(ish) MP3 server.
 *
 * Homepage:
 *   http://www.gnump3d.org
 *
 * Author:
 *  Steve Kemp <skx@tardis.ed.ac.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */

#include "base64.h"



/*
 * Encoding table.
 */
const std::string table = 
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";


/**
 * Constructor.
 */
CBase64::CBase64()
{
}


/**
 * Destructor.
 */
CBase64::~CBase64()
{
}


/**
 * Lookup a character in our 64 byte alphabet.
 */
unsigned int CBase64::LookUp(const char base64Val)  
{
    unsigned int counter;
    for (counter=0; counter<table.length(); counter++)  
    {
	if (table[counter]==base64Val)
	    return(counter);
    }

    /* wasn't a valid charcter, ignore */
    return(99);
}


/**
 * Decode a given string from BASE64 representation.
 * @param text The encoded text.
 * @return The decoded text.
 */
std::string CBase64::decode(std::string text )  
{
    unsigned int convertedVal = 0;
    unsigned int partialVal   = 0;

    unsigned int bits2    = 0;
    unsigned int bits4    = 0;
    unsigned int bits6    = 0;
    unsigned int counter  = 0;
    unsigned int lookedUp = 0;
    std::string result    = "";
    int n = 0;
    

    unsigned int textLength = text.length();
    for (counter=0; counter<textLength && text[counter]!='='; 
	 counter++)  
    {
	lookedUp = LookUp(text[counter]);
	if (lookedUp<70)  
	{
	    n++;
	    switch (n)  
	    {
	    case 1:
		/*
		 * we have no extra bits laying around.
		 * only have 6 bits, so just save them until
		 * we have a read with enough bits
		 */
		bits6 = lookedUp << 2;
		break;
	    case 2:
		/*
		 * we have 6 bits stored.  
		 * our next char has 6 bits.
		 * write out 8 of these bits,
		 * store 4 of these bits
		 */
		partialVal = (lookedUp & 48) >> 4;
		convertedVal = bits6 + partialVal;
		bits4 = lookedUp << 4;
		result +=((unsigned char)convertedVal); 
		break;
	    case 3:
		/*
		 * we have 4 bits stored
		 * our next char has 6 bits
		 * write out 8 of these bits,
		 * store 2 of these bits
		 */
		partialVal = (lookedUp & 60) >> 2;
		convertedVal = bits4 + partialVal;
		bits2 = lookedUp << 6;
		result += ((unsigned char)convertedVal); 
		break;
	    case 4:
		/*
		 * we have 2 bits stored
		 * our nextchar has 6 bits
		 * write out all 8 bits
		 */
		convertedVal = lookedUp + bits2;
		n = 0;
		result += ((unsigned char)convertedVal); 
		break;
	    }
	}  
    }
    
    /* return the encoded string */
    return(result);
}


/**
 * Decode a given string from BASE64 representation.
 * @param text The encoded text.
 * @return The decoded text.
 */
/* static */ std::string CBase64::Decode( std::string text )
{
    CBase64 decoder;
    return( decoder.decode( text  ) );
}
