/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  base64.h - Simple BASE64 decoding.
 *
 *  GNU MP3D - A portable(ish) MP3 server.
 *
 * Homepage:
 *   http://www.gnump3d.org
 *
 * Author:
 *  Steve Kemp <skx@tardis.ed.ac.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */




#ifndef _BASE64_H
#define _BASE64_H 1


#include <string>



/**
 *  This is a utility class for performing base64 decoding.
 *
 *  This class is used to decode the realm authentification header
 * which clients will send to use when password protection is in
 * operation.
 *
 * @author Steve Kemp <A HREF="mailto:skx@tardis.ed.ac.uk">skx@tardis.ed.ac.uk</A>
 * @version $Revision: 1.5 $
 *
 */
class CBase64
{
public:

    /**
     *  Constructor.
     */
    CBase64();

    /**
     * Destructor.
     */
    virtual ~CBase64();

  
    /**
     * Decode a given string from BASE64 representation.
     * @param text The encoded text.
     * @return The decoded text.
     */
    std::string decode( std::string text );


    /**
     * Decode a given string from BASE64 representation.
     * @param text The encoded text.
     * @return The decoded text.
     */
    static std::string Decode( std::string text );

private:

    /**
     * Lookup a character in our 64 byte alphabet.
     * @param base64Val The alphabet character to lookup.
     * @return The position of the character.
     */
    unsigned int LookUp(const char base64Val);

};

#endif /* _BASE64_H */
