#include <stdio.h>
#include <stdlib.h>
#include "iplugin.h"
#include "base64.h"


/**
 *  This plugin is designed to display decoded HTTP realm logins.
 *
 */
class CRealmPlugin: public IPlugin
{
 public:

  CRealmPlugin();
  ~CRealmPlugin();
  
  virtual void processPacket( const char *src, const char *dst, connection *currentConnection );
  virtual std::string getAuthor();
  virtual std::string getDescription();

private:
  std::string getHTTPLine( std::string name, char *buffer );
};


CRealmPlugin::CRealmPlugin()
{
}


CRealmPlugin::~CRealmPlugin()
{
}


std::string CRealmPlugin::getHTTPLine( std::string name, char *buffer )
{
  char *start = NULL;
  char *end   = NULL;

  start = strstr( buffer, name.c_str() );
  if ( start == NULL )
  {
    return "";
  }
  start += strlen( name.c_str() );

  std::string auth;
  int i = 0;
  while( start[i] != '\n' && start[i] != '\r' )
    {
      auth += start[i];
      i++;
    }
  return( auth );
}

void CRealmPlugin::processPacket(const char *src, const char *dst, connection *currentConnection )
{
  std::string auth    = getHTTPLine( "Authorization: Basic ", 
				     currentConnection->buf );
  if ( auth.size() == 0 ) 
  {
     return;
  }

  std::string path    = getHTTPLine( "GET ",
				     currentConnection->buf );
  std::string decoded = CBase64::Decode( auth );
  
  if ( strlen( decoded.c_str() ) )
    {
      printf("GET %s->%s %s\n\t %s\n", src, dst, path.c_str(), decoded.c_str() );
    }
}

std::string CRealmPlugin::getAuthor()
{
  return( "Steve Kemp" );
}

std::string CRealmPlugin::getDescription()
{
	return( "Sniff for HTTP basic realm authentification" );
}


/**
 * Create a new instance of this plugin object.
 */
extern "C" IPlugin *factory()
{
  return new CRealmPlugin();
};
