/*
 * httpcapture application, initially derived fro msome of the sample
 * code included with the usi++ library.
 *
 * Modified to work with plugins.
 *
 */


#include <usi++/usi++.h>
#include <iostream>
#include <string>
#include <vector>

#include "iplugin.h"
#include "plugin.h"


/*
 * All the connections currently in transit.
 */
std::vector< connection* >    conn;



/*
 *  Add a new connection to the global list.
 */
int add_conn(TCP *tcp)
{
    int p = tcp->get_dstport();
		    
    connection *c = new connection;
    memset(c, 0, sizeof(connection));
    c->src = tcp->get_src();
    c->dst = tcp->get_dst();
    c->sport = tcp->get_srcport();
    c->dport = tcp->get_dstport();
    conn.push_back(c);
    return 0;
}


/*
 * Find the given connection from our list.
 */
connection *get_conn(TCP *tcp)
{
    for (int i = 0; i < conn.size(); i++) 
    {
	if (!conn[i])
	    continue;
	if (tcp->get_dst() == conn[i]->dst && tcp->get_src() == conn[i]->src &&
	    tcp->get_dstport() == conn[i]->dport && tcp->get_srcport() == conn[i]->sport)
	    return conn[i];
    }
    return NULL;
}
	

/**
 * Entry point to the code.
 */
int main(int argc, char **argv)
{
    /**
     * Defaults for the command line arguments.
     */
    int debug = 0;
    int list  = 0;
    int help  = 0;
    int vers  = 0;
    int force = 0;

    /*
     * Default plugin directory, and sniffing device.
     */
    std::string plugin_dir = "/usr/local/lib/httpcapture";
    std::string eth_dev    = "eth0";

    for( int i = 1; i < argc; i++ )
    {
	if ( ( strcmp( argv[i], "-h" ) == 0 ) ||
	     ( strcmp( argv[i],  "-H" ) == 0 ) ||
	     ( strcmp( argv[i], "--help" ) == 0 ) )
	{
	    help = 1;
	}
	if ( ( strcmp( argv[i], "-v" ) == 0 ) ||
	     ( strcmp( argv[i],  "-V" ) == 0 ) ||
	     ( strcmp( argv[i], "--version" ) == 0 ) )
	{
	    vers = 1;
	}
	if ( ( strcmp( argv[i], "-f" ) == 0 ) ||
	     ( strcmp( argv[i],  "-F" ) == 0 ) ||
	     ( strcmp( argv[i], "--force" ) == 0 ) )
	{
	    force = 1;
	}
	if ( ( strcmp( argv[i], "-l" ) == 0 ) ||
	     ( strcmp( argv[i],  "-L" ) == 0 ) ||
	     ( strcmp( argv[i], "--list" ) == 0 ) )
	{
	    list = 1;
	}
	if ( ( strcmp( argv[i], "-d" ) == 0 ) ||
	     ( strcmp( argv[i],  "-D" ) == 0 ) ||
	     ( strcmp( argv[i], "--debug" ) == 0 ) )
	{
	    debug = 1;
	    printf(" [Debugging enabled]\n" );
	}
	if ( ( strcmp( argv[i], "-p" ) == 0 ) ||
	     ( strcmp( argv[i], "-P" ) == 0 ) ||
	     ( strcmp( argv[i], "--path" ) == 0 ) )
	{
	    if ( (i+1) < argc )
	    {
		plugin_dir = argv[i+1];
		i++;
	    }
	    else
	    {
		printf("Argument missing for plugin directory.\n" );
		return -1;
	    }
	}
	if ( ( strcmp( argv[i], "-i" ) == 0 ) ||
	     ( strcmp( argv[i], "-i" ) == 0 ) ||
	     ( strcmp( argv[i], "--interface" ) == 0 ) )
	{
	    if ( (i+1) < argc )
	    {
		eth_dev = argv[i+1];
		i++;
	    }
	    else
	    {
		printf("Argument missing for interface to bind upon.\n" );
		return -1;
	    }
	}
    }

    /**
     * Handle any immediate options.
     */
    if ( vers )
    {
	printf("httpcapture - %s by Steve Kemp\n", VERSION_NUMBER );
	printf(" Compiled upon host " HOSTNAME " upon " __DATE__  "\n" );
	return 0;
    }
    if ( help )
    {
	printf("httpcapture - %s by Steve Kemp\n\n", VERSION_NUMBER );
	printf(" Usage: httpcapture [ options ]\n\n" );
	printf("  --debug          Enable extra debugging output.\n" );
	printf("  --force          Don't exit if run by a non-root user.\n" );
	printf("  --help           Show this help\n" );
	printf("  --interface ethN Set the interface to listen upon.\n" );
	printf("  --list           Show all installed plugins.\n" );
	printf("  --path dir       Set an alternate plugin directory.\n" );
	printf("  --version        Show the version number of this application.\n" );
	return 0;
    }


    /*
     * Setup plugin loader singleton.
     */
    CPluginLoader *pl = CPluginLoader::getInstance( plugin_dir, debug );

    if ( list )
    {
	std::vector< CLoadedPlugin *> plugins = pl->getAllPlugins();
	std::vector< CLoadedPlugin * >::iterator i;

	for( i = plugins.begin(); i != plugins.end(); i++ )
	{
	    CLoadedPlugin *found = *i;
	    
	    IPlugin *plug = (*found->factory)();
	    if ( plug )
	    {
		printf("%s by %s\n", 
		       found->description.c_str(),
		       found->author.c_str() );
		
		if ( debug )
		{
		    printf(" [Located in : %s]\n", 
			   found->pluginFilename.c_str() );
		}
	    }
	}
	return 0;
    }

    /*
     * NON-Root users will probably not have permission to bind the socket
     * to promiscuous modes.
     */
    if ( ( 0 != geteuid()) && ( force == 0 ) )
    {
	printf("\n");
	printf("Non-root users will probably not have permission to run this tool.\n" );
	printf("If that isn't the case use '--force' to avoid this error.\n" );
	printf("\n");
	exit( -1 );
    }

    /**
     * Create the sniffing object.
     */
    TCP *sn = new TCP("127.0.0.1");
    sn->init_device( (char *)eth_dev.c_str(), 1, 500);

    char buf[1000];
	
    while (1)
    {
	memset(buf, 0, 1000);
	int l = sn->sniffpack(buf, 1000);
	 
	/* if connection is opened, save characteristics
	 */
	if (sn->get_flags() == TH_SYN) 
	{
	    add_conn(sn);
	} 
	else 
	{
	    /* don't fetch the ACK only packets 
	     */
	    if (sn->get_flags() == TH_ACK)
		continue;

	    connection *c = get_conn(sn);
	    if (!c)
		continue;	
			
	    /* insert data
	     */
	    int j = 0;
	    
	    while (l-- > 0 && c->count < MAXDATALEN) 
		c->buf[c->count++] = buf[j++];
	}

	/*
	 * If the connection is closing then we can handle it
	 * via the plugins.
	 */
	if ( sn->get_flags() & TH_FIN )
	{
	    connection *c = get_conn(sn);
	    if (!c)
		continue;	

	    
	    /*
	     * Get the endpoints of the connection.
	     */
	    char src[1000], dst[1000];

	    UDP dummy("localhost");
	    dummy.set_src(c->src);
	    dummy.set_dst(c->dst);
	    dummy.get_src(0, src, 1000);
	    dummy.get_dst(0, dst, 1000);

	    pl->callPlugin( src, dst, c );
	    delete( c );
	}
    }
     
    return 0;
}

